@@ -10,8 +10,8 @@
 #include "kio_widgets_debug.h"
 #include "kuiserver_interface.h"
 
+#include <KInhibitionJobTracker>
 #include <KJobTrackerInterface>
-#include <KUiServerJobTracker>
 #include <KUiServerV2JobTracker>
 #include <KWidgetJobTracker>
 #include <kio/jobtracker.h>
@@ -24,9 +24,9 @@
 #include <QXmlStreamReader>
 
 struct AllTrackers {
-    KUiServerJobTracker *kuiserverTracker;
     KUiServerV2JobTracker *kuiserverV2Tracker;
     KWidgetJobTracker *widgetTracker;
+    KInhibitionJobTracker *inhibitionTracker;
 };
 
 class KDynamicJobTrackerPrivate
@@ -38,9 +38,9 @@ public:
 
     ~KDynamicJobTrackerPrivate()
     {
-        delete kuiserverTracker;
         delete kuiserverV2Tracker;
         delete widgetTracker;
+        delete inhibitionTracker;
     }
 
     static bool hasDBusInterface(const QString &introspectionData, const QString &interface)
@@ -58,9 +58,9 @@ public:
         return false;
     }
 
-    KUiServerJobTracker *kuiserverTracker = nullptr;
     KUiServerV2JobTracker *kuiserverV2Tracker = nullptr;
     KWidgetJobTracker *widgetTracker = nullptr;
+    KInhibitionJobTracker *inhibitionTracker = nullptr;
     QMap<KJob *, AllTrackers> trackers;
 
     enum JobViewServerSupport {
@@ -96,9 +96,9 @@ void KDynamicJobTracker::registerJob(KJob *job)
     // always add an entry, even with no trackers used at all,
     // so unregisterJob() will work as normal
     AllTrackers &trackers = d->trackers[job];
-    trackers.kuiserverTracker = nullptr;
     trackers.kuiserverV2Tracker = nullptr;
     trackers.widgetTracker = nullptr;
+    trackers.inhibitionTracker = nullptr;
 
     auto useWidgetsFallback = [this, canHaveWidgets, &trackers, job] {
         if (canHaveWidgets) {
@@ -124,6 +124,15 @@ void KDynamicJobTracker::registerJob(KJob *job)
         return;
     }
 
+    if (!job->property("kio_no_inhibit_suspend").toBool()) {
+        if (!d->inhibitionTracker) {
+            d->inhibitionTracker = new KInhibitionJobTracker();
+        }
+
+        trackers.inhibitionTracker = d->inhibitionTracker;
+        trackers.inhibitionTracker->registerJob(job);
+    }
+
     const QString kuiserverService = QStringLiteral("org.kde.kuiserver");
 
     if (!d->jobViewServerWatcher) {
@@ -163,7 +172,12 @@ void KDynamicJobTracker::registerJob(KJob *job)
             d->jobViewServerSupport = KDynamicJobTrackerPrivate::Error;
         }
 
-        QDBusConnection::sessionBus().connect(kuiserverService, QStringLiteral("/JobViewServer"), QStringLiteral("org.kde.kuiserver"), QStringLiteral("requiresJobTrackerChanged"), this, SLOT(handleRequiresJobTrackerChanged(bool)));
+        QDBusConnection::sessionBus().connect(kuiserverService,
+                                              QStringLiteral("/JobViewServer"),
+                                              QStringLiteral("org.kde.kuiserver"),
+                                              QStringLiteral("requiresJobTrackerChanged"),
+                                              this,
+                                              SLOT(handleRequiresJobTrackerChanged(bool)));
     }
 
     if (d->jobViewServerSupport == KDynamicJobTrackerPrivate::V2Supported) {
@@ -176,19 +190,9 @@ void KDynamicJobTracker::registerJob(KJob *job)
         return;
     }
 
-    // No point in trying to set up V1 if calling the service above failed.
-    if (d->jobViewServerSupport != KDynamicJobTrackerPrivate::Error) {
-        if (!d->kuiserverTracker) {
-            d->kuiserverTracker = new KUiServerJobTracker();
-        }
-
-        trackers.kuiserverTracker = d->kuiserverTracker;
-        trackers.kuiserverTracker->registerJob(job);
-    }
-
     // If kuiserver isn't available or it tells us a job tracker is required
     // create a widget tracker.
-    if (d->jobViewServerSupport == KDynamicJobTrackerPrivate::Error) {
+    if (d->jobViewServerSupport == KDynamicJobTrackerPrivate::Error || d->jobViewServerSupport == KDynamicJobTrackerPrivate::V2NotSupported) {
         useWidgetsFallback();
     }
 }
@@ -205,13 +209,9 @@ void KDynamicJobTracker::unregisterJob(KJob *job)
     }
 
     const AllTrackers &trackers = it.value();
-    KUiServerJobTracker *kuiserverTracker = trackers.kuiserverTracker;
     KUiServerV2JobTracker *kuiserverV2Tracker = trackers.kuiserverV2Tracker;
     KWidgetJobTracker *widgetTracker = trackers.widgetTracker;
-
-    if (kuiserverTracker) {
-        kuiserverTracker->unregisterJob(job);
-    }
+    KInhibitionJobTracker *inhibitionTracker = trackers.inhibitionTracker;
 
     if (kuiserverV2Tracker) {
         kuiserverV2Tracker->unregisterJob(job);
@@ -221,6 +221,10 @@ void KDynamicJobTracker::unregisterJob(KJob *job)
         widgetTracker->unregisterJob(job);
     }
 
+    if (inhibitionTracker) {
+        inhibitionTracker->unregisterJob(job);
+    }
+
     d->trackers.erase(it);
 }
 
